
%% Disclaimer
%{

The information in this document is currently being subjected to peer 
review. Mention of trade names or commercial products does not constitute 
endorsement or recommendation for use by the developers.

The code here must be used at the user's own risk. 

Neither Industrial Economics, Inc., nor the program authors can assume 
responsibility for model operation, output, interpretation or usage.

The creators of this program have used their best efforts in preparing this 
code. It is not absolutely guaranteed to be error free. The author/programmer 
makes no warrantees, expressed or implied, including without limitation 
warrantees of merchantability or fitness for any particular purpose. No 
liability is accepted in any event for any damages, including accidental or 
consequential damages, lost of profits, costs of lost data or programming 
materials, or otherwise in connection with or arising out of the use of 
this program.

%}

% Road processing
%  - Calculate Capacity
%  - Calculate Redundancy Factor
%  - Identify Obstacles

clear

% options and parameters
searchRadius = 5; % radius for redundancy calculation (half of average trip length)

home.shpOrig = 'State Runs orig/'; % location of original shape files
home.shpJoin = 'State Runs _origJoin/'; % crosswalk between original and segmented
home.shpSeg = 'State Runs proc_mdpt/'; % location of segmented shape files

names.states = {'Alabama';'California';'Connecticut';'Delaware';'District';
  'Florida';'Georgia';'Louisiana';'Maine';'Maryland';'Massachusetts';'Mississippi';
  'NewHampshire';'NewJersey';'NewYork';'NorthCarolina';'Oregon';'Pennsylvania';
  'RhodeIsland';'SouthCarolina';'Texas';'Virginia';'Washington'};

num.states = length(names.states);

% HPMS variable names
HPMSvars = {
 ... database name, struct field name, default values when field is missing
  'urban_code' 'urban_code' 99999
  'speed_limi' 'speed_limi' 0
  'through_la' 'through_la' 2
  'peak_lanes' 'peak_lanes' 1
  'pct_peak_s' 'pct_peak_s' 0
  'pct_peak_c' 'pct_peak_c' 0
  'signal_typ' 'signal_typ' 0 % sets all to signaled except FC1&2 or those with stop signs
  'pct_green_' 'pct_green_' 0
  'stop_signs' 'stop_signs' 0 % sets none to stop signs except FC7 roads 
  'lane_width' 'lane_width' 0
  'shoulder_w' 'shoulder_w' 0
  'terrain_ty' 'terrain_ty' 1
  'LENGTH'     'len'        []
  'f_system'   'FC'         []
  'aadt'       'AADT'       []
	'aadt_combi' 'AADT_COMBI' []
	'aadt_singl' 'AADT_SINGL' []
  'MID_X'      'lon'        []
  'MID_Y'      'lat'        []
  };

% variables from the joined dataset
HPMSvarsJ = {'Seg_ID'     'SegID' %text
             'widening_o' 'widening_Obst' %text
             'fldInd'     'flood'
             'LENGTH'     'len'
             'through_la' 'lanes'
             'lane_width' 'lane_width'
             'aadt'       'AADT'
             'aadt_combi' 'AADT_COMBI'
             'aadt_singl' 'AADT_SINGL'
             'shoulder_t' 'shoulder_type'
             'shoulder_w' 'shoulder_widthR'
             'shoulder_1' 'shoulder_widthL'
             'median_wid' 'median_width'
             'urban_code' 'urban_code'
             };

% variables from the segment dataset
SegVars = {
   ... database name, struct field name
   'Seg_ID' 'segID'
   'LENGTH_1' 'len'
   'W_AADT' 'AADT'
   'BuffFld' 'flood'
   'POINT_X' 'lon'
   'POINT_Y' 'lat'
   'F_SYSTEM' 'FC'
   };

% preallocate
AADTavg = nan(num.states,7,4);
AADTzeros = AADTavg;
ErrorMiles = nan(num.states,1); % miles with heavy vehicle traffic greater than 99% of total, assumed to be an error
Capacity = cell(num.states,1);
Rfactor = nan(num.states,1); % redundancy factor AADT / Capacity
fieldMissing.FC1_2 = zeros(num.states,size(HPMSvars,1));
fieldMissing.FC3_7 = zeros(num.states,size(HPMSvars,1));
allRfactors = [];
Sall = [];
c=0;

tic % start clock
for st = 1:num.states
  
  ST = names.states{st};
  
%   if strcmp(ST,'Georgia')
%     continue
%   end
  
  if ~exist([home.shpOrig 'tmp/'],'dir')
    mkdir([home.shpOrig 'tmp/'])
  end
  
  if ~exist([home.shpJoin 'tmp/'],'dir')
    mkdir([home.shpJoin 'tmp/'])
  end
  
  %% load data
  
  switch ST
    case 'California' % california is too big
      
      % loads data for California (too big to load from the dbf file
      % directly)
      load([home.shpOrig 'California_FC1_2.mat']);
      CaliforniaFC12.aadt_combi = CaliforniaFC12.aadt_conbi; % spelling mistake in raw data
      CaliforniaFC37 = load([home.shpOrig 'California_FC3_7.mat']);
      CaliforniaFC37.aadt_combi = CaliforniaFC37.aadt_conbi; % spelling mistake in raw data
      
      % pull the variables we need 
      for i = 1:size(HPMSvars,1)
        H.(HPMSvars{i,2}) = [CaliforniaFC12.(HPMSvars{i,1})(2:end)
                             CaliforniaFC37.(HPMSvars{i,1})(2:end)];
      end
      for i = [7:9 12]
        H.(HPMSvars{i,2})(:) =  HPMSvars{i,3};
      end
      
      clear CaliforniaFC12 CaliforniaFC37
      
      unzip([home.shpJoin ST '1_2.zip'],[home.shpJoin 'tmp/'])
      unzip([home.shpJoin ST '3_7.zip'],[home.shpJoin 'tmp/'])
      FC12 = readtable([home.shpJoin 'tmp/' ST '1_2/FC1_2_origJoin.csv']);
      FC12.Properties.VariableNames{9} = 'aadt_combi';
      FC37_n = readtable([home.shpJoin 'tmp/' ST '3_7/FC3_7_NorCal_origJoin.csv']);
      FC37_n.Properties.VariableNames{10} = 'aadt_combi';
      FC37_s = readtable([home.shpJoin 'tmp/' ST '3_7/FC3_7_SoCal_origJoin.csv']);
      FC37_s.Properties.VariableNames{13} = 'aadt_combi';
      for i = 1:size(HPMSvarsJ,1)
        Hj.(HPMSvarsJ{i,2}) = [FC12.(HPMSvarsJ{i,1}) 
                               FC37_n.(HPMSvarsJ{i,1})
                               FC37_s.(HPMSvarsJ{i,1})];
      end
      Hj.flood(isnan(Hj.flood)) = false;
      save([home.shpJoin 'CaliforniaJoin.mat'],'Hj')
      
      load([home.shpJoin 'CaliforniaJoin.mat'],'Hj')
      
      
    otherwise
      
      %-- loads original HPMS data
      unzip([home.shpOrig ST '1_2.zip'],[home.shpOrig 'tmp/'])
      unzip([home.shpOrig ST '3_7.zip'],[home.shpOrig 'tmp/'])
      [data.FC1_2,text.FC1_2] = xlsread([home.shpOrig 'tmp/FC1_2.dbf']);
      [data.FC3_7,text.FC3_7] = xlsread([home.shpOrig 'tmp/FC3_7.dbf']);
      
      % counting fields
      numFields(st,1) = size(text.FC1_2,2);
      numFields(st,2) = size(text.FC3_7,2);
      
      % fixes spelling mistake in raw data for some States
      ind = strcmp(text.FC1_2(1,:),'aadt_conbi');
      if sum(ind)>0
        text.FC1_2{1,ind} = 'aadt_combi'; 
      end
      
      % pull the variables we need (FC groups need to be seperated because
      % some States have different formats between the two, e.g., GA)
      for i = 1:size(HPMSvars,1)
        for v = {'FC1_2' 'FC3_7'}
          ind = strcmpi(text.(v{:})(1,:),HPMSvars{i,1});
          if sum(ind)>0
            x.(v{:}) = data.(v{:})(:,ind);
          else % if missing set to default value
            x.(v{:}) = ones(size(data.(v{:}),1),1) .* HPMSvars{i,3};
            fieldMissing.(v{:})(st,i) = 1;
          end
        end
        H.(HPMSvars{i,2}) = [x.FC1_2;x.FC3_7];
      end
      
      % set these to default (only a couple states have them)
      for i = [7:9 12]
        H.(HPMSvars{i,2})(:) =  HPMSvars{i,3};
      end
      
      %-- orignal data joined to segments
      unzip([home.shpJoin ST '1_2.zip'],[home.shpJoin 'tmp/'])
      unzip([home.shpJoin ST '3_7.zip'],[home.shpJoin 'tmp/'])
      [dataJ.FC1_2,textJ.FC1_2] = xlsread([home.shpJoin 'tmp/' ST '1_2/FC1_2_origJoin.dbf']);
      [dataJ.FC3_7,textJ.FC3_7] = xlsread([home.shpJoin 'tmp/' ST '3_7/FC3_7_origJoin.dbf']);
      
      % fixes spelling mistake in raw data for some States
      ind = strcmp(textJ.FC1_2(1,:),'aadt_conbi');
      if sum(ind)>0
        textJ.FC1_2{1,ind} = 'aadt_combi'; 
      end
      
      % pull the variables we need (FC groups need to be seperated because
      for i = 1:size(HPMSvarsJ,1)
        for v = {'FC1_2' 'FC3_7'}
          ind = strcmpi(textJ.(v{:})(1,:),HPMSvarsJ{i,1});
          if sum(ind)>0
            if i<=3 % text variables
              x.(v{:}) = textJ.(v{:})(2:end,ind);
            else
              x.(v{:}) = dataJ.(v{:})(:,ind);
            end
          else 
            keyboard
          end
        end
        Hj.(HPMSvarsJ{i,2}) = [x.FC1_2;x.FC3_7];
      end
      
      Hj.flood(cellfun(@isempty,Hj.flood)) = {'0'};
      Hj.flood = str2double(Hj.flood);
  
  end
  
  % Assume these roads are 2 lanes
  Hj.lanes(Hj.lanes==0) = 2;
  
  %-- load segmented data
  unzip([home.shpSeg ST '1_2.zip'],[home.shpSeg 'tmp/'])
  unzip([home.shpSeg ST '3_7.zip'],[home.shpSeg 'tmp/'])
  [data12,text12] = xlsread([home.shpSeg 'tmp/' ST '1_2/FC1_2_proc_mdpt.dbf']);
  [data37,text37] = xlsread([home.shpSeg 'tmp/' ST '3_7/FC3_7_proc_mdpt.dbf']);
  if size(data12,2) < size(text12,2)
    data12(:,size(text12,2)) = 0; % fixes issue when all BuffFld is zero
  end
  S = table();
  for i = 1:size(SegVars,1)
    S.(SegVars{i,2}) = [data12(:,strcmpi(text12(1,:),SegVars{i,1}))
      data37(:,strcmpi(text37(1,:),SegVars{i,1}))];
  end
  S.segID = [text12(2:end,strcmpi(text12(1,:),'Seg_ID'))
    text37(2:end,strcmpi(text37(1,:),'Seg_ID'))];
 
  %% Heavy Vehicle (Single and combination) calculations
  
  % remove any with over 99% heavy truck
  AADT = [H.AADT H.AADT_COMBI H.AADT_SINGL]; %[AADT, COMBI, SINGL]
  ind = sum(AADT(:,2:3),2) >= 0.99*AADT(:,1)  & AADT(:,1)>0;
  AADT(ind,2:3) = nan;
  ErrorMiles(st) = sum(H.len(ind));
  
  for fc = 1:7 % loop over functional classes
    indfc = fc == H.FC; % index for FC
    indnon0 = AADT(:,1)>0; % index for non-zero
    ind = indfc & indnon0; % index for calculating AADT
    ind0 = ~indnon0 & indfc; % index for zeros
    if sum(ind)~=0
      % length-weighted AADT
      xx = nansum((AADT(ind,:) .* H.len(ind))) ./ nansum(H.len(ind));
      if length(xx)>1
        AADTavg(st,fc,1:3) = xx;
      else
        AADTavg(st,fc,1) = xx;
      end
      % length in miles
      AADTavg(st,fc,4) = nansum(H.len(ind)) ./ 5280; % in miles
      % zeros
      if sum(ind0)>0
        % count of zeros
        AADTzeros(st,fc,1:3) = nansum(ind0,1);
        % miles of zeros
        AADTzeros(st,fc,4) = nansum(H.len(ind0)) ./ 5280; % in miles
      end
    else
      % flag when FC is missing from dataset
      AADTavg(st,fc,:) = -9999;
    end
  end
  
  % average to use if missing values (calculations over entire matrix
  % because it's easier)
  AADTavg(AADTavg<0) = nan;
  AADTavg(AADTavg(:,4)==0,1:3) = nan;
  HVper = sum(AADTavg(:,:,2:3),3,'omitnan') ./ AADTavg(:,:,1);
  HVper(isnan(HVper)) = 0;
  
  %% Capacity calculation
    % Based on 
    %
    % The capacity of a roadway facility is the maximum reasonable hourly 
    % rate at which vehicles can be expected to transverse a point or a uniform 
    % section of lane or roadway during a given time period under prevailing roadway, 
    % traffic, and control conditions. (HPMS field manual)
    %
    % item numbers refer to the HPMS field manual designation
  
  % prealocate
  Cap = zeros(length(H.FC),1);
    
  % used to keep track of segments already calculated
  Calculated = ones(length(H.FC),1)==0;
  
  % Lanes used in calculations for more than one type
  % represents single direction number of lanes (for one-way capacity calculations)  
  Lanes = H.peak_lanes; % item 10
  Lanes(Lanes==0) = H.through_la(Lanes==0)./2; % many are zero, use one-way through lane value
  Lanes(Lanes<1) = ceil(Lanes(Lanes<1)); % set one-way to 1
  Lanes(Lanes==0 | isnan(Lanes)) = 1; % assume the rest are 1, can't picture a zero-lane road
  
  %-- freeways (Fig 2 & 3)
  % FC 1&2 are freeways (criteria on pg 9) "If functional system is one or 
  % two, and full access control exists, then the section is a freeway."
  ind = H.FC==1 | H.FC==2; % freeway indices
  f_lw = H.lane_width(ind); % item 34
  f_lw(f_lw==0) = 12; % default value from Table 3
  f_rlc = H.shoulder_w(ind);% item 38
  f_rlc(f_rlc==0|isnan(f_rlc)) = 10; % default value from Table 3
  FFS = 75.4 - f_lw - f_rlc; % fig 3
  HV = nansum(AADT(ind,2:3),2) ./ AADT(ind,1);% frac of combi & singl
  HV(isnan(HV)) = 0; % cases where AADT is zero and AADT from combi and singl are zero
  Cap(ind) = ((2200 + 10*(min(70,FFS)-50)) ./ (1+HV)) .* Lanes(ind); % fig 2
  Calculated(ind) = true; % keep track so it is not recalculated
  
  if sum(isnan(Cap))>0
    keyboard
  end
  
  %--Stop-controlled highways
  % "If stop signs exist, then the section is stop controlled" (pg 9) [also
  % assume FC 7 are stop controlled]
  ind1 = (H.stop_signs>0 | H.FC==7) & Lanes==1 & ~Calculated; % item 32 one lane
  ind2 = (H.stop_signs>0 | H.FC==7) & Lanes>1 & ~Calculated; % more than one lane
  Cap(ind1) = 1200; % fig 6
  Cap(ind2) = 1500; % fig 6
  Calculated(ind1|ind2) = true; % keep track so it is not recalculated
  
  %--Signalized highways.
  % "If signals exist, then the section is signalized." (pg 9) [also assume
  % FC=6 are all signaled]
  ind = (H.signal_typ~= 5 | H.FC==6) & ~Calculated; % item 29 (5=No signal systems exist)
  g_C = H.pct_green_(ind); % item 30
  g_C(g_C==0) = 50;
  Cap(ind) = g_C/100 .* Lanes(ind) * 1900; % fig 5
  Calculated(ind) = true; % keep track so it is not recalculated
  
  if sum(isnan(Cap))>0
    keyboard
  end
  
  %--multilane highways
  % "If through lanes are greater than or equal to four, the section is
  % multilane." (pg 9)
  ind = H.through_la>=4 & ~Calculated;% item 7, through lanes
  FFS = H.speed_limi(ind) + 5;% item 14 + 5 (many are missing this!!)
  FFS(FFS<=5 & H.urban_code(ind)==99999) = 65 + 5;% default values for rural from table 3
  FFS(FFS<=5 & H.urban_code(ind)~=99999) = 55 + 5;% default values for urban from table 3
  BaseCap = 1000 + 20 .* FFS; % base capacity (Fig 4)
  BaseCap(FFS>60) = 2200; % fig 4 cap
  PT = H.pct_peak_s(ind) + H.pct_peak_c(ind);% item 23 + item 25
  ET = zeros(sum(ind),1); % set up terrain adjustment
  ET(H.urban_code(ind)~=99999) = 1.5; % all urban roads
  ET(H.terrain_ty(ind)==1) = 1.5; % item 44, terrain type 1 is "level"
  ET(H.terrain_ty(ind)==2) = 2.5; % item 44, terrain type 1 is "Rolling"
  ET(H.terrain_ty(ind)==3) = 4.5; % item 44, terrain type 1 is "Mountainous"
  ET(ET==0 & H.urban_code(ind)==99999) = 2.5;% default values for rural from Table 3
  ET(ET==0 & H.urban_code(ind)~=99999) = 1.5;% default values for urban from Table 3
  f_hv = 1 ./ (1 + PT .* (ET-1)); % adjustment for heavy vehicles
  Cap(ind) = BaseCap .* f_hv .* Lanes(ind); % multilane capacity (Fig 4)
  Calculated(ind) = true; % keep track so it is not recalculated
  
  if sum(isnan(Cap))>0
    keyboard
  end
  
  %-- Rural 2- and 3-lane highways with no traffic control (pg 6)
  ind = H.through_la<4 & H.through_la>1 & H.urban_code==99999 & ~Calculated; % 
  Cap(ind) = 1490; % use default from NCHRP 825 method (pg 6)
  Calculated(ind) = true; % keep track so it is not recalculated
  
  %--all remaining (based on suggestion on pg 9)
  ind = ~Calculated; 
  Cap(ind) = Lanes(ind) * 1900; 
  
  %% keep capacity (units are vehicles / hour)
  Capacity{st} = Cap;
  H.Cap = Cap;
  
  % state-wide redundancy factors for QC 
  Rfactor(st,1) = nansum(AADT(:,1) .* H.len) / 24 / (nansum(Cap .* H.len));
  Rfactor(st,2) = nansum(AADT(:,1) .* H.len) / nansum(H.len); % H.length-weighted AADT
  Rfactor(st,3) = nansum(Cap .* H.len) ./ nansum(H.len); % length-weighted capacity
  
  %% calculate redundancy factor for all segments in the processed data
  floodi = find(S.flood); % only for segments in the flood zone
  N = length(floodi);
  if N==0
    keyboard
  end
  % preallocate
  vars = {'VMTinRad' 'CapinRad' 'Rfactor' 'AADT_COMBI' 'AADT_SINGL' 'lanes' 'lane_width' 'shoulder_type' ...
    'shoulder_widthR' 'shoulder_widthL' 'median_width' 'widening_obst' 'urban' 'missingJoin'};
  for v = vars
    S.(v{:}) = zeros(height(S),1);
  end
  S.shoulder_type = cell(height(S),1);
  S.widening_obst = cell(height(S),1);
  % loop over segments
  for segi = 1:N
    
    % find in search radius
    seg = floodi(segi);
    dist = deg2km(distance(S.lat(seg),S.lon(seg),H.lat,H.lon)) * 0.621371; % distances - convert to miles 
    ind = dist <= max(searchRadius,(S.len(seg)/5280/2)); % search radius
    
    % find segment capacity
    fcind = find(H.FC == S.FC(segi));
    [a,b] = min(dist(fcind));
    segCap = Cap(fcind(b)) * H.len(fcind(b));
    
    % R factor
    S.VMTinRad(seg) = sum(H.AADT(ind) .* H.len(ind),'omitnan');
    S.CapinRad(seg) = sum(Cap(ind) .* H.len(ind),'omitnan');
    S.Rfactor(seg) = sum(H.AADT(ind) .* H.len(ind),'omitnan') / 24 / (sum(Cap(ind) .* H.len(ind),'omitnan') - segCap);
    
    % add joined segment data
    segID = S.segID{seg};
    ind = strcmp(Hj.SegID,segID);
    if sum(Hj.flood(ind))>0
      % traffic
      S.AADT_COMBI(seg) = sum(Hj.flood(ind) .* Hj.AADT_COMBI(ind) .* Hj.len(ind),'omitnan') ./ sum(Hj.flood(ind) .* Hj.len(ind),'omitnan');
      S.AADT_SINGL(seg) = sum(Hj.flood(ind) .* Hj.AADT_SINGL(ind) .* Hj.len(ind),'omitnan') ./ sum(Hj.flood(ind) .* Hj.len(ind),'omitnan');
      % lanes
      S.lanes(seg) = sum(Hj.flood(ind) .* Hj.lanes(ind) .* Hj.len(ind),'omitnan') ./ sum(Hj.flood(ind) .* Hj.len(ind),'omitnan');
      S.lane_width(seg) = sum(Hj.flood(ind) .* Hj.lane_width(ind) .* Hj.len(ind),'omitnan') ./ sum(Hj.flood(ind) .* Hj.len(ind),'omitnan');
      % shoulders
      shoulders = Hj.shoulder_type(ind & Hj.flood);
      if max(shoulders)==7% if curb
        S.shoulder_type{seg} = 'curb';
      elseif max(shoulders)==6 % earth
        S.shoulder_type{seg} = 'earth';
      elseif max(shoulders)>1 %Paved/Gravel
        S.shoulder_type{seg} = 'Paved/Gravel';
      else % none=1
        S.shoulder_type{seg} = 'none';
      end
      S.shoulder_widthR(seg) = sum(Hj.flood(ind) .* Hj.shoulder_widthR(ind) .* Hj.len(ind),'omitnan') ./ sum(Hj.flood(ind) .* Hj.len(ind),'omitnan');
      S.shoulder_widthL(seg) = sum(Hj.flood(ind) .* Hj.shoulder_widthL(ind) .* Hj.len(ind),'omitnan') ./ sum(Hj.flood(ind) .* Hj.len(ind),'omitnan');
      % median
      S.median_width(seg) = sum(Hj.flood(ind) .* Hj.median_width(ind) .* Hj.len(ind),'omitnan') ./ sum(Hj.flood(ind) .* Hj.len(ind),'omitnan');
      %obstacles 
      obs = Hj.widening_Obst(ind & Hj.flood==1);
      obsi = ismember(obs,{'A' 'B' 'C' 'D' 'E' 'F' 'G'});
      if sum(obsi)>1
        S.widening_obst{seg} = [obs{:}]; % lists all obstacle codes
      elseif isempty([obs{:}])
        S.widening_obst{seg} = NaN;
      else
        S.widening_obst{seg} = 'None';
      end
      ucode = Hj.urban_code(ind & Hj.flood==1);
      S.urban(seg) = max(ucode)~=99999; % if no rural roads in segment, is urban
    elseif sum(ind)>0 % if no segments are designated as flooded but segments exist in join, use full segment 
      S.missingJoin(seg) = 1;
      % traffic
      S.AADT_COMBI(seg) = sum(Hj.AADT_COMBI(ind) .* Hj.len(ind),'omitnan') ./ sum(Hj.len(ind),'omitnan');
      S.AADT_SINGL(seg) = sum(Hj.AADT_SINGL(ind) .* Hj.len(ind),'omitnan') ./ sum(Hj.len(ind),'omitnan');
      % lanes
      S.lanes(seg) = sum(Hj.lanes(ind) .* Hj.len(ind),'omitnan') ./ sum(Hj.len(ind),'omitnan');
      S.lane_width(seg) = sum(Hj.lane_width(ind) .* Hj.len(ind),'omitnan') ./ sum(Hj.len(ind),'omitnan');
      % shoulders
      shoulders = Hj.shoulder_type(ind);
      if max(shoulders)==7% if curb
        S.shoulder_type{seg} = 'curb';
      elseif max(shoulders)==6 % earth
        S.shoulder_type{seg} = 'earth';
      elseif max(shoulders)>1 %Paved/Gravel
        S.shoulder_type{seg} = 'Paved/Gravel';
      else % none=1
        S.shoulder_type{seg} = 'none';
      end
      S.shoulder_widthR(seg) = sum(Hj.shoulder_widthR(ind) .* Hj.len(ind),'omitnan') ./ sum(Hj.len(ind),'omitnan');
      S.shoulder_widthL(seg) = sum(Hj.shoulder_widthL(ind) .* Hj.len(ind),'omitnan') ./ sum(Hj.len(ind),'omitnan');
      % median
      S.median_width(seg) = sum(Hj.median_width(ind) .* Hj.len(ind),'omitnan') ./ sum(Hj.len(ind),'omitnan');
      %obstacles 
      obs = Hj.widening_Obst(ind);
      obsi = ismember(obs,{'A' 'B' 'C' 'D' 'E' 'F' 'G'});
      if sum(obsi)>1
        S.widening_obst{seg} = [obs{:}]; % lists all obstacle codes
      elseif isempty([obs{:}])
        S.widening_obst{seg} = NaN;
      else
        S.widening_obst{seg} = 'None';
      end
      ucode = Hj.urban_code(ind);
      S.urban(seg) = max(ucode)~=99999; % if no rural roads in segment, is urban
    else % if everything is missing, use default values
      S.missingJoin(seg) = 2;
      S.lanes(seg) = 2;
      S.AADT_COMBI(seg) = HVper(st,S.FC(seg)) * S.AADT(seg) / 2; % split equally between COMBI & SINGL
      S.AADT_SINGL(seg) = HVper(st,S.FC(seg)) * S.AADT(seg) / 2;
      S.lane_width(seg) = mean(Hj.lane_width(Hj.lane_width~=0));
      S.shoulder_type{seg} = 'none'; 
      S.shoulder_widthR(seg) = 0; % indicates no shoulder
      S.shoulder_widthL(seg) = 0; % indicates no shoulder
      S.median_width(seg) = 0; % indicates no median
      S.widening_obst{seg} = NaN; % indicates missing
      S.urban(seg) = 0; % set to rural
    end

  end
  save(['Mat Files\Road Segments\AllSegments_' ST '.mat'],'S','H','Hj')
  Sf = S(S.flood==1,:);
  Sall = [Sall;Sf];
  state(c+1:(height(Sf)+c)) = repmat({ST},height(Sf),1);
  c = c + height(Sf);
    
  % delete upacked file
  i=0;
  while i < 10 % sometimes computer is fussy
    try
      rmdir([home.shpOrig 'tmp\'],'s')
      rmdir([home.shpSeg 'tmp\'],'s')
      rmdir([home.shpJoin 'tmp\'],'s')
      i=10;
    catch err
      i = i + 1;
      pause(2)
      disp([num2str(i) '... trying again | ' err.message])
    end
  end

  fprintf('%d. %s | %0.2f min \n',st,ST,toc/60)
  
  clear H Hj S data text
  
end

Sall.State = state';
save('AllSegmentsFlooded.mat','Sall')

% make state average portion of heavy vehicles
clear key
key.state = names.states;
save('HeavyVehicleFrac.mat','HVper','key')

%% postprocessing of Rfactor


% process Rfactors
% string together
Sall = [];
c=0;
state = cell(28973,1);
for st = 1:num.states
  ST = names.states{st};
  if ~strcmp(ST,'Georgia')
    load(['Rfactor\Rfactor_' ST '_default.mat'],'S')
    S = S(S.flood==1,:);
    Sall = [Sall;S];
    state(c+1:(height(S)+c)) = repmat({ST},height(S),1);
    c = c + height(S);
  end
end
Sall.State = state;

save('RedundancyFactor.mat','Sall')

% histogram of redundancy factors

figure(45), clf
histogram(Sall.Rfactor,0:.01:.6)
xlabel('Rerouting Effectiveness Factor')
ylabel('Number of segments (out of ~29,000)')
savePlot(gcf,'plots\','Rerouting Effectivness Factor Histogram',400)
close

%% references

%{

1	'ET_ID'
2	'YEAR_RECOR'
3	'STATE_CODE'
4	'ROUTE_ID'
5	'BEGIN_POIN'
6	'END_POINT'
7	'ROUTE_NUMB'
8	'ROUTE_NAME'
9	'ROUTE_QUAL'
10	'ROUTE_SIGN'
11	'F_SYSTEM'
12	'FACILITY_T'
13	'URBAN_CODE'
14	'NHS'
15	'STRAHNET'
16	'OWNERSHIP'
17	'TRUCK'
18	'THROUGH_LA'
19	'PEAK_LANES'
20	'COUNTER_PE'
21	'SHOULDER_T'
22	'SHOULDER_W'
23	'SHOULDER_1'
24	'LANE_WIDTH'
25	'MEDIAN_TYP'
26	'MEDIAN_WID'
27	'ACCESS_CON'
28	'WIDENING_O'
29	'WIDENING_P'
30	'HOV_LANES'
31	'HOV_TYPE'
32	'SPEED_LIMI'
33	'TOLL_CHARG'
34	'TOLL_ID'
35	'TOLL_TYPE'
36	'AADT'
37	'AADT_COMBI'
38	'AADT_SINGL'
39	'PCT_PEAK_C'
40	'PCT_PEAK_S'
41	'K_FACTOR'
42	'DIR_FACTOR'
43	'IRI'
44	'PSR'
45	'SURFACE_TY'
46	'CRACKING_P'
47	'FAULTING'
48	'RUTTING'
49	'STRUCTURE_'
50	'YEAR_LAST_'
51	'YEAR_LAST1'
52	'IS_SAMPLE'
53	'SAMPLE_ID'
54	'EXPANSION_'
55	'National_R'
56	'Shape_Leng'
57	'OrigRd_ID'
58	'ET_X'
59	'ET_Y'
60	'ET_IDP'
61	'ET_M'
62	'POINT_X'
63	'POINT_Y'


      %{
      [data, text] = dbfread([home.shp lower(ST) '_project\' lower(ST) '_project.dbf']);
%       [data,text] = xlsread([home.shp lower(ST) '_project\' lower(ST) '_project.dbf']);
      save([home.shp lower(ST) '_project\' lower(ST) '_project.mat'],'-v7.3','data','text')
      load([home.shp lower(ST) '_project\' lower(ST) '_project.mat'])
      for v = text
        try
          X.(v{:}) = cell2mat(data(:,strcmp(v{:},text)));
        catch
          X.(v{:}) = data(:,strcmp(v{:},text));
        end
      end
      save([home.shp lower(ST) '_project\' lower(ST) '_project_unpacked.mat'],'-v7.3','-struct','X')
      len = cell2mat(data(:,strcmp('LENGTH',text))); % length (ft)
      AADT(:,1) = cell2mat(data(:,strcmp('aadt',text))); %[AADT, COMBI, SINGL]
      AADT(:,2) = cell2mat(data(:,strcmp('aadt_conbi',text)));
      AADT(:,3) = cell2mat(data(:,strcmp('aadt_singl',text)));
      FC = cell2mat(data(:,strcmp('f_system',text))); % functional class
      save([home.shp lower(ST) '_project\' lower(ST) '_project_vars.mat'],'len','AADT','FC')
      %}
%       load([home.shp lower(ST) '_project/' lower(ST) '_project_vars.mat'])
%       H = load([home.shp lower(ST) '_project/' lower(ST) '_project_unpacked.mat']);

%}
