
%% Disclaimer
%{

The information in this document is currently being subjected to peer 
review. Mention of trade names or commercial products does not constitute 
endorsement or recommendation for use by the developers.

The code here must be used at the user's own risk. 

Neither Industrial Economics, Inc., nor the program authors can assume 
responsibility for model operation, output, interpretation or usage.

The creators of this program have used their best efforts in preparing this 
code. It is not absolutely guaranteed to be error free. The author/programmer 
makes no warrantees, expressed or implied, including without limitation 
warrantees of merchantability or fitness for any particular purpose. No 
liability is accepted in any event for any damages, including accidental or 
consequential damages, lost of profits, costs of lost data or programming 
materials, or otherwise in connection with or arising out of the use of 
this program.

%}

% Traffic Delays and Costs
%  - Calculate delays and costs
%  - Calculate reasonably anticipated adaptation effects
%  - Simulate direct adaptation response

clear

% Version name (appends to name of results files)
% Version = 'v3_withoutConstructDelays';
Version = 'v4_AdaptFinal';

% projections of traffic based on population and GDP
WithGrowth = 1;

% option to force sea walls
ForceSeaWalls = 0;

% requires at least FloodDepth cm above minor level to be classified as a flood
floodDepth = 0; %(cm) 

% sensitivity analysis on delay costs (requested by reviewer #1, 06/24/2020)
costRatio = 1;

% directories
home.home = 'D:\Dropbox\EPA SLR\03 Analysis\10 Nuisance Floods and Traffic\';
% home.home = 'C:\Users\CFant\Dropbox\EPA SLR\03 Analysis\10 Nuisance Floods and Traffic\';
home.road = [home.home '01 Road and Traffic\'];
home.tide = [home.home '02 Tide Gauge\'];
home.cost = [home.home '04 Costs\'];
home.adapt = [home.home '05 Adaptation\'];
home.input = [home.cost 'input\'];
home.output = [home.cost 'output\'];

%% - Parameters and Options

% delay costs
passCost = 20.40 * 1.67; % $20.40 per person hour (DOT 2016) by 1.67 people / vehicle (FHWA 2017)
HVcost = 64 + 35; % $/hour, O&M($64) + medium time sensitive cargo ($35) (DOT 2016)

% adaptation costs and other direct adaptation params
% Raised Profile
adrpCost = 125000./13; % to raise profile for 1 ft of width for 1 ft height for one mile (C = (H[ft] * Lanes*laneWidth[ft] * Length[mi])
adrpVarFrCost = 1; % all costs vary with height
adrpEmb = 1760; % embankment costs in square ft cross sectional for one mile: based on $9 / sq yard
adrpDelay = 6*24./13; % delay from construction (hours/ft width/ft raised)
adrpOffset = 10; % accounts for missing regular maintenance cycle
adLife(1) = 20; % life of raised profile (in years), on regular maintenance cycle of 20 years
% Sea wall
adswCost = 1000*3.28*1.376; % Cost per meter for 1m high sea wall. C*H^2*L (all in m) (based on Neumann et al. (2015), adjusted to $2015
adLife(2) = 50; % life of sea wall (in years)
adOMcost = [.03 .03]; % O & M costs for [raising_profile, sea_walls]
% misc adaptation parameters
adLenMin = .5; % minimum length in miles
Sfactors = [1 2 4 10]; % BC ratio threshold to trigger adaptation
adHorizon = 10; % adaptation horizon

if ForceSeaWalls
  Version = [Version '_ForceSeaWalls'];
end

if WithGrowth
  Version = [Version '_WithGrowth'];
end

if floodDepth ~= 0
  Version = [Version '_Depth' num2str(floodDepth) 'cm'];
end

% set to no delay cost
if strcmp('v3_withoutConstructDelays',Version)
  adrpDelay = 0;
end

% add delay cost sensitivity to name
if costRatio~=1
  Version = [Version '_DelayCostSensitivity_' num2str((costRatio-1)*100) 'perc'];
  passCost = passCost*costRatio;
  HVcost = HVcost*costRatio;
end

% add maintenance delay to name
if adrpOffset~=10
  Version = [Version '_Mdelay' num2str(adrpOffset) 'years'];
end

%% Load Data

% load tide gauge information
fn = [home.tide 'CWF Copy of Projected_1920_2100_HTfloods_hrs.xls'];
warning off
TGinfo = readtable(fn,'sheet','Names');
TGinfo = TGinfo(TGinfo.Include==1,:);
warning on
num.gauge = height(TGinfo);

% load SLR
SLR = load([home.input 'SLRsweetEtAl.mat']);
names.SLRscen = {'low' 'intlow' 'int' 'inthigh' 'high' 'extreme'};
num.SLRscen = length(names.SLRscen);
num.cnty = length(SLR.key.county);
years = 2000:2100;
names.eras = 2030:20:2100;
names.eraYrs = {2020:2039 2040:2059 2060:2079 2080:2099};
num.years = length(years);
num.eras = length(names.eras);
names.cnty = SLR.key.county;
% interpolate from 10-year to every year
sealevel = zeros(num.years+50,num.SLRscen,num.cnty);
for i = 1:6
  sealevel10 = [zeros(num.cnty,1) SLR.SLR(:,:,i,1)]; % "1" assumes Medium uncertainty
  % allow to interpolate beyond 2100 for adaptation look ahead
  sealevel(:,i,:) = interp1(2000:10:2100,sealevel10',years(1):years(end)+50,'linear','extrap');
end
% RCP weights from Sweet et al. 2017
RCPw = ...
  [0.14500	0.47500	0.36250	0.01450	0.00225	0.00075
  0.02000	0.41500	0.47350	0.08350	0.00600	0.00200];


% Load REF_State_County
REF_StateCounty = readtable([home.input 'REF_StateCounty.xlsx']);

% tide gauge links to county
tgassign = readtable([home.tide 'TideGauge_Assignment.xlsx'],'Sheet','FinalAssignment');

% hours at at cm by tide gauge (for hours projections)
HRS = load([home.tide 'HoursAtLevel.mat']);
levels = -6:.01:2;

% references, names, and numbers
REF_StateCounty = innerjoin(REF_StateCounty,tgassign,'LeftKeys','County_FIPS',...
  'RightKeys','FIPS','RightVariables','FinalAssignment');
REF_StateCounty.State{34} = 'District'; % match name of file for traffic data
names.states = unique(REF_StateCounty.State,'stable');
num.states = length(names.states);

names.FCs = {'FC 1-2' 'FC 3-5' 'FC 6-7'};
num.FCgroup = {1:2 3:5 6:7};
num.FCs = length(names.FCs);

names.vers = {'No Buffer' '200 ft Buffer' '500 ft Buffer'};
num.vers = length(names.vers);
names.verSuff = {'bufferBTC_0'   'WAADT_Diss_joinFIPS' 'FldInd'
  'erase_bufferBTC_200_diss' 'erase_bufferBTC_200_diss' 'BuffFld'
  'bufferBTC_500' 'erase_bufferBTC_500' 'BuffFld'};

names.adapt = {'No Adapt' 'Redundancy' 'Ancillary' 'Redundancy + Ancillary' 'Long-term Decisions' 'All Autonomous' };

% Values for filling
Fill = readtable([home.road 'StateLengthWeightedAADT.xlsx'],'Sheet','Missing','Range','E62:G85');

% road redundancy factor for autonomous adaptation
load([home.road 'RedundancyFactor.mat']);

% heavy vehicle data
load([home.road 'HeavyVehicleFrac.mat'],'HVper')
HVper(isnan(HVper)) = 0;

% ancillary protection
load([home.adapt 'Property Protection\src\AncillaryProtection.mat'],'status01')

% population and GDP for projecting traffic
if WithGrowth
  POP = readtable([home.input 'ICLUS_v2_UN_Probabilistic_Median.xlsx'],'Sheet', 'Output');
  ind = ismemberInd(POP.FIPS,names.cnty);
  POP = POP(ind,:);
  POP.yearly = interp1([1990:10:2010 2015:5:2100],table2array(POP(:,8:end))',years)';
  GDP.gdp = xlsread([home.input 'FINAL CIRA 2.0 GDP Projections.xlsx'],'2000 Start','D4:D104');
  GDP.ratio = GDP.gdp ./ GDP.gdp(years==2016);
  POP.ratio = POP.yearly ./ POP.yearly(:,years==2016);
else
  POP.ratio = ones(num.cnty,num.years);
  GDP.ratio = ones(num.years,1);
end

% direct adaptation
names.adaptMethods = {'Raise Profile' 'Armor'};
%   adMult = 1 + 0:.25:2; % mutliplier so it is more expensive after the first
for ss = Sfactors
  names.adapt{end+1} = ['Direct Adaptation S' num2str(ss)];
end
num.adapt = length(names.adapt);

tic %start clock

ver = 2; % select buffer version

% preallocate and general pre-loop set up
prevState = []; % keep track of state (avoid loading for every county)
SegmentData = [];
istate = 0; % state count
Delays = zeros(num.years,num.FCs,num.cnty,num.SLRscen,num.adapt);
DelaysRamps = Delays; % delays from ramps only
Costs = Delays; % keep costs (2014 USD)
CapitalCosts = Delays;
CostsHV = Delays;
segProtect = Delays;
ProtectedCost = Delays;
unProtectedCost = Delays;
segSelections = zeros(num.years,num.FCs,num.cnty,num.SLRscen,num.adapt,2);
trafProtect = segSelections;
HoursOrig = nan(num.years,num.SLRscen,num.cnty); HoursAll = HoursOrig;
missing = zeros(num.cnty,3);
xx = nan(num.states,1);
StateStats = table(names.states,unique(REF_StateCounty.State_Abbreviation,'stable'),xx,xx,xx,xx,xx,xx,xx,xx,xx,...
  'VariableNames',{'State' 'State_Abr' 'NumSeg12' 'NumSeg345' 'NumSeg67' 'Miles12' 'Miles345' 'Miles67' 'MilesZero12' 'MilesZero345' 'MilesZero67'});
xx = nan(num.cnty,1);
CountyStats = table(REF_StateCounty.County,REF_StateCounty.State_Abbreviation,REF_StateCounty.County_FIPS,xx,xx,xx,xx,xx,xx,xx,xx,xx,...
  'VariableNames',{'County' 'State_Abr' 'FIPS' 'NumSeg12' 'NumSeg345' 'NumSeg67' 'Miles12' 'Miles345' 'Miles67' 'MilesZero12' 'MilesZero345' 'MilesZero67'});

% loops
for cnty = 1:num.cnty
  
  %-- find and load tide gauge data
  stationName = REF_StateCounty.FinalAssignment{cnty};
  gaugeID = TGinfo.NOAAID(strcmp(TGinfo.StationName,stationName));
  
  % load hours (from Billy Sweet sent on 5/23/2019)
  fn = [num2str(gaugeID) '_19yr_kopp.mat'];
  load([home.tide 'from B Sweet\observations\' fn])
  
  % hours flooded per simulation year
  hours = 24*365*...convert from years to hours
    [pdf_cdf.low_hr pdf_cdf.intlow_hr pdf_cdf.int_hr ...
    pdf_cdf.inthigh_hr pdf_cdf.high_hr pdf_cdf.extreme_hr];
  
  HoursOrig(:,:,cnty) = hours;
  
  % apply SLR projections to hours at level
  hours = zeros(num.years,num.SLRscen);
  MHHWyi = ones(num.years,num.SLRscen);
  gauge = HRS.key.info.NOAAID == gaugeID;
  for scen = 1:num.SLRscen
    %     CDF = (24*365)*(1-pdf_cdf.cdf_hr);
    %     hrs = interp1(pdf_cdf.freq_hr-info.Minor_Flood_Level(gauge),CDF,levels);
    sl = round((sealevel(1:num.years,scen,cnty)-floodDepth)*10); % convert to mm
    lev = round(-HRS.levels'*1e3);
    [~,order] = ismember(sl,lev);
    hours(:, scen) = HRS.hrsAtLevel(order,gauge);
    % when does MHHW reach minor level?
    ml = round(TGinfo.Minor_Flood_Level(strcmp(TGinfo.StationName,stationName))*1e3); %Minor level in mm
    yi = findClosestInd(sl,ml);
    rampdown = 0.9:-.1:0;
    if yi < num.years
      ind = yi:min(num.years,yi+9);
      MHHWyi(ind,scen) = rampdown(1:length(ind)); % ramp down
      if length(ind)==10
        MHHWyi(min(yi+10,num.years):end,scen) = 0; % zero after ramp down
      end
    end
  end
  HoursAll(:,:,cnty) = hours;
  hours(hours<0)=0;
  %   CDF = (24*365)*(1-pdf_cdf.cdf_hr);
  
  %% load traffic impacted
  State = REF_StateCounty.State{cnty};

  if ~strcmp(State,prevState) % only load data if state changes (saves run time)
    
    % keep previous state name
    prevState = State;
    % state count
    istate = istate + 1;
    
    % load output from GIS analysis on road segmentation, traffic, and
    % flood zone (from Natalie W. dated 06/12/2019)
    State(State==' ') = []; % remove spaces
    fn = [home.input 'State Runs ' names.verSuff{ver,1} '\' State '1_2\FC1_2_' names.verSuff{ver,2} '.dbf'];
    [data12,text12] = xlsread(fn);
    fn = [home.input 'State Runs ' names.verSuff{ver,1} '\' State '3_7\FC3_7_' names.verSuff{ver,2} '.dbf'];
    [data37,text37] = xlsread(fn);
    if size(data12,2) < size(text12,2)
      data12(:,size(text12,2)) = 0; % fixes issue when all BuffFld is zero
    end
    
    % make a table
    xx = [text12(2:end,[find(strcmp(text12(1,:),'FIPS')) find(strcmp(text12(1,:),'Seg_ID'))])
      text37(2:end,[find(strcmp(text37(1,:),'FIPS')) find(strcmp(text37(1,:),'Seg_ID'))])];
    traffic = table(xx(:,1),xx(:,2),'VariableNames',{'FIPS' 'Seg_ID'});
    
    % unpack data [1.Join_Count	2.TARGET_FID	3.F_SYSTEM	4.Seg_ID	5.LENGTH_1	6.SUM_VMT
    %7.W_AADT	8.FldInd	9.FIPS	10.RampInd] <- NOTE indices are not always the
    % same for all States
    for v = {'Fsystem'   'VMT'      'AADT'    'flood'              'length' % variable names
        'F_SYSTEM'  'SUM_VMT'  'W_AADT'  names.verSuff{ver,3} 'LENGTH_1'} % names in dbf file
      
      traffic.(v{1}) = [data12(:,strcmpi(text12(1,:),v{2}))
        data37(:,strcmpi(text37(1,:),v{2}))];
      
    end
    traffic.flood = traffic.flood==1; % convert to logical
    traffic.length = traffic.length ./ 5280; % convert from feet to miles
    
    % add ramp indices (only in FC 1&2)
    traffic.ramp = zeros(height(traffic),1);
    traffic.ramp(1:size(data12,1)) = data12(:,strcmpi(text12(1,:),'RampInd'));
    
    clear data12 data37 text12 text37
    
    %-- counts and miles
    % number of segemnts and miles
    for fc = 1:num.FCs
      ind = ismember(traffic.Fsystem,num.FCgroup{fc}) & traffic.flood;
      if sum(ind)
        fcstr = num2str(num.FCgroup{fc}); fcstr(fcstr==' ') = [];
        StateStats.(['NumSeg' fcstr])(istate) = sum(ind);
        StateStats.(['Miles' fcstr])(istate) = sum(traffic.length(ind),'omitnan');
        ind0 = ind & traffic.AADT==0;
        if sum(ind0)>1
          StateStats.(['MilesZero' fcstr])(istate) = sum(traffic.length(ind0),'omitnan');
        end
      end
    end
    
    %-- Fill in missing FC 6 & 7
    fill = table2array(Fill(strcmp(Fill.State,State),2:3));
    for fc = 6:7
      ind = traffic.Fsystem==fc & traffic.AADT==0;
      traffic.AADT(ind) = fill(fc-5);
    end
    
    % join redundancy factor and other characteristics with road segments
    load([home.road 'Mat Files\Road Segments\AllSegments_' State '.mat'],'S')
    rtvars = S.Properties.VariableNames(10:end);
    traffic = innerjoin(traffic,S,...
      'LeftKeys','Seg_ID','RightKeys','segID',...
      'RightVariables',rtvars);
    traffic.Rfactor(isnan(traffic.Rfactor)) = 0;
    traffic.Rfactor(isinf(traffic.Rfactor)) = 1;
    
    % fill in flooded indicator for repeated segment IDs
    IDs = unique(traffic.Seg_ID(traffic.flood));
    ind = ismember(traffic.Seg_ID,IDs);
    traffic.flood(ind) = 1==1;
    
    % default missing road width with average class width for state
    for fc = 1:7
      fcind = traffic.Fsystem==fc;
      traffic.lane_width(traffic.lane_width==0 & fcind) = ...
        mean(traffic.lane_width(traffic.lane_width>0 & fcind),'omitnan');
    end
    
    % correct when heavy traffic is same as AADT (issue in GA and MS)
    traffic.AADT_SINGL(traffic.AADT_SINGL>=traffic.AADT) = 0;
    traffic.AADT_COMBI(traffic.AADT_COMBI>=traffic.AADT) = 0;
    traffic.AADT_SINGL(isnan(traffic.AADT_SINGL)) = 0;
    traffic.AADT_COMBI(isnan(traffic.AADT_COMBI)) = 0;
    
    % double traffic for NY and NJ FC1-2
    switch State
      case {'NY' 'NJ'}
        traffic.AADT(traffic.Fsystem==1 | traffic.Fsystem==2) = ...
          traffic.AADT(traffic.Fsystem==1 | traffic.Fsystem==2)*2;
    end
    
    % load length of flooded area along road for adaptation costs
    fn = [home.road 'State Runs IntFF_Diss\' State '1_2\FC1_2_IntFF_Diss.dbf'];
    [data12,text12] = xlsread(fn);
    fn = [home.road 'State Runs IntFF_Diss\' State '3_7\FC3_7_IntFF_Diss.dbf'];
    [data37,text37] = xlsread(fn);
    floodLen = table([text12(2:end,1);text37(2:end,1)],[data12;data37],'VariableNames',text12(1,:));
    tt = outerjoin(traffic,floodLen,'Type','left','Keys','Seg_ID','MergeKeys',true);
    if height(tt)==height(traffic)
      traffic = tt;
    else
      keyboard
    end
    traffic.Properties.VariableNames{end} = 'FloodLen_ft';
    
    fprintf(' %d. %d/23 %0.2f min \n',ver,istate,toc/60)
    
  end
  
  %% County Results
  
  % pop and gdp projections for projecting traffic
  pop = POP.ratio(cnty,:);
  gdp = GDP.ratio';
  
  %--Ancillary Protection - Links between NCPM grids and roads
  ST = REF_StateCounty.State_Abbreviation{cnty};
  CY = REF_StateCounty.County_NoSpaces{cnty};
  [~,~,roadJoin12] = xlsread([home.adapt 'county join roads/' ST '_' CY '_FC1_2_gridJoin.dbf' ]);
  roadJoin12 = cell2table(roadJoin12(2:end,:),'VariableNames',roadJoin12(1,:));
  ind = roadJoin12.BuffFld==1 & roadJoin12.RampInd~=1;
  roadETIDs12 = roadJoin12(ind,...
    [find(strcmp(roadJoin12.Properties.VariableNames,'ET_ID'))
    find(strcmp(roadJoin12.Properties.VariableNames,'Seg_ID'))]);
  [~,~,roadJoin37] = xlsread([home.adapt 'county join roads/' ST '_' CY '_FC3_7_gridJoin.dbf' ]);
  roadJoin37 = cell2table(roadJoin37(2:end,:),'VariableNames',roadJoin37(1,:));
  ind = roadJoin37.BuffFld==1;
  roadETIDs37 = roadJoin37(ind,...
    [find(strcmp(roadJoin37.Properties.VariableNames,'ET_ID'))
    find(strcmp(roadJoin37.Properties.VariableNames,'Seg_ID'))]);
  roadETIDs = [roadETIDs12; roadETIDs37];
  try
    [~,~,floodJoin] = xlsread([home.adapt 'county join flood/' ST '_' CY '_mrg.dbf' ]);
    colET = strcmp(floodJoin(1,:),'ET_ID');
    colfl = strcmp(floodJoin(1,:),'FldPln');
    floodETIDs = cell2mat(floodJoin(find(cell2mat(floodJoin(2:end,colfl))==1)+1,colET));
    % ET IDs and seg_IDs that are flooded
    [ind,order] = ismember(roadETIDs.ET_ID,floodETIDs);
    roadETIDs = roadETIDs(ind,:); % final dataset of flooded ETIDs with road segment ID
  catch
    disp(['Missing Flood Join: ' ST '_' CY]);
    roadETIDs = [];
  end
  
  %-- loop over functional classes
  for fc = 1:num.FCs
    
    % pull out traffic only for county and FC group
    ind = strcmp(traffic.FIPS,REF_StateCounty.County_FIPS{cnty}) & ismember(traffic.Fsystem,num.FCgroup{fc}) & traffic.ramp~=1;
    traf = traffic(ind,:);
    
    % preal new variables
    traf.HVtraffPerHr = zeros(height(traf),1); 
    traf.PassTraffPerHr = zeros(height(traf),1);
    for scen = 1:num.SLRscen
      traf.(['ProtYear_S' num2str(scen)]) = zeros(height(traf),1); 
    end
    
    %-- Estimate delays for segments in the flood zone & county stats
    if sum(ind)>0 && sum(traf.flood) % skip if no roads in county are in flood zone
      
      % heavy vehicle traffic costs ($/hour flooded)
      traf.HVtraffPerHr(traf.flood) = (traf.AADT_COMBI(traf.flood) + traf.AADT_SINGL(traf.flood)) ./ 24;
      % passenger vehicle traffic costs ($/hour flooded)
      traf.PassTraffPerHr(traf.flood) = (traf.AADT(traf.flood) ./ 24) - traf.HVtraffPerHr(traf.flood);
      % cost per hour of delay
      K = (traf.HVtraffPerHr(traf.flood)*HVcost.*gdp + traf.PassTraffPerHr(traf.flood)*passCost.*pop); 
            
      %-delays
      Delays(:,fc,cnty,:,1) = sum(traf.PassTraffPerHr(traf.flood) .* pop...
                                     + traf.HVtraffPerHr(traf.flood) .* gdp,1,'omitnan')' .* hours;
      
      %- Apply redundancy factor 
      Delays(:,fc,cnty,:,2) = sum(...
                                     (traf.PassTraffPerHr(traf.flood) .* pop ...
                                     + traf.HVtraffPerHr(traf.flood) .* gdp) ...
                                     .* traf.Rfactor(traf.flood),1,'omitnan')' .* hours;
      
      %- Protection from actions to protect property
      protected = zeros(num.years,num.SLRscen);
      protCostAnc = zeros(num.years,num.SLRscen);
      hrsAnc = zeros(size(K,1),num.years,num.SLRscen);
      for scen = 1:num.SLRscen
        status = status01{cnty,scen};
        hrsAnc1 = repmat(hours(:,scen)',[size(K,1) 1]); % (for costs) putting before continue statement
        if isempty(status) || isempty(roadETIDs)
          Delays(:,fc,cnty,scen,3) = Delays(:,fc,cnty,scen,1);
          Delays(:,fc,cnty,scen,4) = Delays(:,fc,cnty,scen,2);
          continue
        end
        protETIDs = innerjoin(table(status(:,1),status(:,3:end)),roadETIDs,'LeftKeys','Var1','RightKeys','ET_ID');
        protETIDs(cellfun(@isnumeric,protETIDs.Seg_ID),:) = []; % removes seg_IDs with NaN
        protSegs = innerjoin(protETIDs,traf(traf.flood,:),...
          'RightKeys','Seg_ID','LeftKeys','Seg_ID');
        % remove duplicates
        [~,ia] = unique(protSegs.Seg_ID);
        protSegs = protSegs(ia,:);
        if ~isempty(protSegs)

          % xx is a binary where 1 is protected
          xx = reshape(permute(repmat(protSegs.Var2,1,1,10),[1 3 2]),[height(protSegs) 100])';
          xx = xx([1 1:end],:); % repeat year 2001
          % ancillary only
          protDelay = xx .* (protSegs.PassTraffPerHr .* pop ...
                             + protSegs.HVtraffPerHr .* gdp)' ...
                             .* hours(:,scen);
          Delays(:,fc,cnty,scen,3) = Delays(:,fc,cnty,scen,1) - sum(protDelay,2,'omitnan');
          % with rerouting
          protDelay2 = xx .* ((protSegs.PassTraffPerHr .* pop ...
                             + protSegs.HVtraffPerHr .* gdp) ...
                             .* protSegs.Rfactor)' ...
                             .* hours(:,scen);
          Delays(:,fc,cnty,scen,4) = Delays(:,fc,cnty,scen,2) - sum(protDelay2,2,'omitnan');

          % record segements that are protected and year and add to traff
          % table
          [~,B] = max(protSegs.Var2,[],2);
          NCPMyears = repmat((2005:10:2095),height(protSegs),1);
          protInd = sum(~xx(2:end,:),1)'+1;
          protInd(protInd==num.years) = 0; % setting cells that do not protect to zero
          protSegs.(['ProtYear_S' num2str(scen)]) = protInd;
          ind1 = ismemberInd(traf.Seg_ID,protSegs.Seg_ID);
          for seg = 1:length(ind1) % sometimes seg IDs repeat (if FC changes in a segment), this loop deals with that
            ind2 = strcmp(protSegs.Seg_ID,traf.Seg_ID(ind1(seg)));
            traf.(['ProtYear_S' num2str(scen)])(ind1(seg)) = protSegs.(['ProtYear_S' num2str(scen)])(ind2);
          end

        end
        
        % make hrsAnc, which is the hours flooded with zeros when protected
        xx = traf.(['ProtYear_S' num2str(scen)])(traf.flood); % index of first year protected
        for seg = 1:size(K,1)
          if xx(seg)>0
            hrsAnc1(seg,xx(seg):end) = 0;% hours with zeros when protected
          end
        end
        hrsAnc(:,:,scen) = hrsAnc1; % used for adaptation as well
          
      end
      
      %-longterm decisions (at MHHW level, drivers no longer use the road)
      Delays(:,fc,cnty,:,5) = squeeze(Delays(:,fc,cnty,:,1)) .* MHHWyi;
      Delays(:,fc,cnty,:,6) = squeeze(Delays(:,fc,cnty,:,4)) .* MHHWyi;
      
      %-costs
      % No Adapt costs
      Costs(:,fc,cnty,:,1) = sum(K,1)' .* hours;
      % rerouting
      Costs(:,fc,cnty,:,2) = sum(K .* traf.Rfactor(traf.flood),1)' .* hours;
      % ancillary
      Costs(:,fc,cnty,:,3) = sum(K .* hrsAnc,1);
      % ancillary & rerouting
      Costs(:,fc,cnty,:,4) = sum(K .* traf.Rfactor(traf.flood) .* hrsAnc,1);
      
      % county stats
      fcstr = num2str(num.FCgroup{fc}); fcstr(fcstr==' ') = [];
      CountyStats.(['NumSeg' fcstr])(istate) = sum(ind & traffic.flood); % number of segments
      CountyStats.(['Miles' fcstr])(istate) = sum(traf.length(traf.flood),'omitnan'); % miles
      ind0 = ind & traffic.flood & traffic.AADT==0;
      if sum(ind0)>1
        CountyStats.(['MilesZero' fcstr])(istate) = sum(traffic.length(ind0),'omitnan');
      end
      
      % bookkeeping - segment data
      for scen = 1:num.SLRscen
        nn = ['ProtYear_S' num2str(scen)];
        if ~sum(strcmp(nn,traf.Properties.VariableNames))
          traf.(nn) = nan(height(traf),1); % present so it is a variable even if not filled
        end
      end
      sgdt = traf(traf.flood==1,[1:3 5 7 9:end]);
      Kr = K .* sgdt.Rfactor; % cost per hour of delay with Rfactor
      xx = permute(RCPw(2,:),[1 3 2]);
      sgdt.DelaysRCP85_2050 = sum(hours(51,:) .* ...
                              (sgdt.PassTraffPerHr .* pop(:,51)...
                              + sgdt.HVtraffPerHr .* gdp(:,51))...
                              .* RCPw(2,:),2);
      sgdt.RCP85_2030 = sum(hours(31,:) .* K(:,31) .* RCPw(2,:),2);
      sgdt.RCP85_2050 = sum(hours(51,:) .* K(:,51) .* RCPw(2,:),2);
      sgdt.RCP85_2070 = sum(hours(71,:) .* K(:,71) .* RCPw(2,:),2);
      sgdt.RCP85_2090 = sum(hours(91,:) .* K(:,91) .* RCPw(2,:),2);
      sgdt.RCP85_RAA_2030 = sum(hrsAnc(:,31,:) .* Kr(:,31) .* xx,3);
      sgdt.RCP85_RAA_2050 = sum(hrsAnc(:,51,:) .* Kr(:,51) .* xx,3);
      sgdt.RCP85_RAA_2070 = sum(hrsAnc(:,71,:) .* Kr(:,71) .* xx,3);
      sgdt.RCP85_RAA_2090 = sum(hrsAnc(:,91,:) .* Kr(:,91) .* xx,3);
      sgdt.S50_2030 = hours(31,2) .* K(:,31);
      sgdt.S50_2050 = hours(51,2) .* K(:,51);
      sgdt.S50_2070 = hours(71,2) .* K(:,71);
      sgdt.S50_2090 = hours(91,2) .* K(:,91);
      sgdt.S100_2030 = hours(31,3) .* K(:,31);
      sgdt.S100_2050 = hours(51,3) .* K(:,51);
      sgdt.S100_2070 = hours(71,3) .* K(:,71);
      sgdt.S100_2090 = hours(91,3) .* K(:,91);

      
      %% Direct adaptation
      
      % estimate delay costs over horizon (Benefit)
      segCost = permute(hours,[3 1 2]) .* Kr; % segment cost
      check = squeeze(sum(segCost,1)) - squeeze(Costs(:,fc,cnty,:,4));
      if sum(check(:))>1e-6
%         keyboard
      end 
      horCost = pvfix(0.03,adHorizon,segCost); % cost over the planning horizon
      floodLen = max(sgdt.FloodLen_ft./5280,adLenMin); % flood length in miles
      
      % raising the profile cost and B/C
      Hrp = permute(sealevel(adLife(1)+1:adLife(1)+num.years,:,cnty),[3 1 2])*3.28./100; % assume height is SLR @ adlife (convert from cm to ft)
      raisepCost = floodLen .* ...
                  (adrpCost .* Hrp .* sgdt.lanes.* sgdt.lane_width ...add more base to road
                  + adrpEmb*Hrp.^2 ...embankment costs (45 deg angle * 2 sides of road)
                    ); 
      constDelC = floodLen .* (Kr .* adrpDelay .* Hrp .* sgdt.lanes.* sgdt.lane_width);% construction delay cost
      BCrp = horCost ./ ...% benefit cost ratio
        (raisepCost.* (1+pvfix(.03,adHorizon,adOMcost(1))) + constDelC); % add O&M costs over planning horizon)
      
      % sea wall cost and B/C
      Hsw = permute(sealevel(adLife(2)+1:adLife(2)+num.years,:,cnty),[3 1 2])./100; % assume height is 2100 SLR (convert from cm to m)
      Hsw(Hsw<.5) = 0.5; % don't build a sea wall if under half meter, dicsussed with Jim N. on Oct 1, 2019
      seawallCost = adswCost .* floodLen*1609.34 .* Hsw.^2; % 1609.34 converts from mi to m
      BCsw = horCost ./ ... % benefit cost ratio
        (seawallCost .* (1+pvfix(.03,adHorizon,adOMcost(2)))); % add O&M costs over planning horizon)
      
      % loop over S factors, scenarios and segments
      adCosts = cat(4,raisepCost+constDelC,seawallCost);
      for iter = 1:length(Sfactors)
        
        Sfactor = Sfactors(iter); % B/C ratio to trigger adaptation
        % binary of BC ratios meeting criteria to build
        adBinrp = BCrp>Sfactor;
        adBinsw = BCsw>Sfactor;
        % which to choose?
        sumBin = cat(2,sum(adBinrp,2),sum(adBinsw,2));
        [~,select] = max(sumBin,[],2); % raised profile = 1, sea wall = 2
        adCostSeg = segCost; % set to segment costs (will be updated if adapt)
        adCapCostSeg = zeros(size(segCost));
        % add adaptation cost with a loop
        for scen = 1:num.SLRscen
          % preallocate segment decisions
          if iter==3
            sgdt.(['adMethod_' names.SLRscen{scen}]) = zeros(size(adCostSeg,1),1);
            sgdt.(['adYear_' names.SLRscen{scen}]) = zeros(size(adCostSeg,1),1);
          end
          for seg = 1:size(adCostSeg,1)
            % conditionally force sea wall in areas where there is likely
            % to be development or there are widening obstacles noted in database
            if (sgdt.urban(seg) && sgdt.lanes(seg)>=4) ... if urban and 4 or more lanes, force sea wall
                || ~(strcmp(sgdt.widening_obst{seg},'None') || sum(isnan(sgdt.widening_obst{seg}))) ... % or if there are obstacles, force sea wall
                || ForceSeaWalls % option to force all to be sea walls
              sel = 2;
              select(seg,1,scen) = sel; % for bookkeeping only
              bin = adBinsw(seg,:,scen);
            else % otherwise, free to choose
              sel = select(seg,1,scen); % raised profile = 1, sea wall = 2
              bin = max(adBinrp(seg,:,scen),adBinsw(seg,:,scen));
            end
            
            % add in adaptation costs
            if sum(bin) % if protected
              
              % build years
              [~,indAd] = max(bin,[],2);
              if sel==1 % delay to mid-cycle if raising profile
                indAd = min(num.years,indAd+adrpOffset);
              end
              
              % distribute adpatation costs over horizon and include
              % rebuild years
              buildYears = (indAd:adLife(sel):num.years); % actual build years
              yi = (indAd:adLife(sel):num.years) + (0:(adHorizon*2)-1)'; % year index for spreading capital costs over horizon
              
              % remove delay costs, yay adapted!
              adCostSeg(seg,indAd:end,scen) = 0; 
              
              % add capital costs
              for y = 1:length(buildYears)
                if y==1 % first build, capital cost is spread over 10 years
                  capCost = adCosts(seg,buildYears(y),scen,sel)/(adHorizon*2); 
                else % for builds after first, subtract previous cost (build on top)
                  capCost = adCosts(seg,buildYears(y),scen,sel)/(adHorizon*2) - prevCost;
                end
                ys = yi(:,y);
                ys(ys>num.years) = [];% trim the tail
                adCostSeg(seg,ys,scen) = capCost;
                prevCost = capCost;
                % keep capital costs
                adCapCostSeg(seg,ys,scen) = capCost;
              end
              
              % add O & M Costs to capital costs
              OMvec = zeros(1,num.years); 
              for y = buildYears % loop over actual build years
                if sel == 1 % remove construction delays when raising profile
                  OM = floodLen(seg) .* (adrpCost .* Hrp(y) .* sgdt.lanes(seg).* sgdt.lane_width(seg) + adrpEmb*Hrp(y).^2);
                elseif sel==2 % sea walls 
                  OM = adCosts(seg,y,scen,sel);
                end
                % build O&M cost vector
                OMvec(y:y+adLife(sel)) = OM * adOMcost(sel);
              end
              adCostSeg(seg,:,scen) = adCostSeg(seg,:,scen) + OMvec(1:length(1:num.years)); % O&M Costs

              % bookkeeping
              trafProtect(indAd,fc,cnty,scen,6+iter,sel) = sgdt.AADT(seg) + ...
                trafProtect(indAd,fc,cnty,scen,6+iter,sel);
              if iter==3 % keep segment adaptation option for S=4 only
                sgdt.(['adMethod_' names.SLRscen{scen}])(seg) = sel;
                sgdt.(['adYear_' names.SLRscen{scen}])(seg) = years(indAd);
                
              end
              ProtectedCost(:,fc,cnty,scen,6+iter) = adCostSeg(seg,:,scen)' + ...
                                                      ProtectedCost(:,fc,cnty,scen,6+iter);
            else
              select(seg,1,scen) = 0; % 0 if not protected
              unProtectedCost(:,fc,cnty,scen,6+iter) = segCost(seg,:,scen)' + ...
                                                      unProtectedCost(:,fc,cnty,scen,6+iter);
            end % if protected
          end % seg loop
          
          % keep track of choice
          segSelections(indAd,fc,cnty,scen,6+iter,1) = sum(select(:,:,scen)==1,1);
          segSelections(indAd,fc,cnty,scen,6+iter,2) = sum(select(:,:,scen)==2,1);
        
          inddc = strcmp(sgdt.Seg_ID,'2610SG');
          if inddc
            dcSel(:,iter) = squeeze(select(inddc,:,:));
            dcCost(:,:,iter) = squeeze(adCostSeg(inddc,:,:));
          end
           
        end % scen loop
        
        % sum costs over segments
        Costs(:,fc,cnty,:,6+iter) = sum(adCostSeg,1,'omitnan');
        CapitalCosts(:,fc,cnty,:,6+iter) = sum(adCapCostSeg,1,'omitnan');
        
      end % s factor iterations
      
      % record User and const cost
      sgdt.raisepUserCostPerft = Kr(:,15) .* adrpDelay .* sgdt.lanes .* sgdt.lane_width; % construction delay cost for base year (2015)
      sgdt.raisepConstCostPerft = adrpCost .* sgdt.lanes .* sgdt.lane_width;
      
      % keep all segment data
      SegmentData = [SegmentData;sgdt];
      
    else
      missing(cnty,fc) = 1; % flag if missing
    end % if flooded
    
  end % functional class loop
  
end % county loop

% Decided to exclude Ramps because they are a small portion of total delays
% and carry more uncertainty since ramps may be elevated 
Delays(isnan(Delays)) = 0;
DelaysRamps(isnan(DelaysRamps)) = 0;
% Delays = Delays - DelaysRamps;
Delays(Delays<0) = 0;

% save all data as mat file
key = names;
key.dims = {'year' 'functional class' 'county' 'SLR scenario' 'Adaptation'};
save([home.output 'Nuis_Flood_Results_' Version '.mat'],'Delays','Costs',...
  'SegmentData','HoursOrig','HoursAll','trafProtect','segSelections',...
  'ProtectedCost','unProtectedCost','CapitalCosts','key')

writetable(SegmentData,[home.output 'SegmentData_' Version '.xlsx'])

